@@ -1,3 +1,26 @@
+/*
+ * This file is part of edicom application
+ *
+ * Copyright (C) 2010 Sugnan Prabhu S <sugnan.prabhu@gmail.com>
+ *
+ * Contact: Sugnan Prabhu S <sugnan.prabhu@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301 USA
+ *
+ */
 #include<fcntl.h>
 #include<unistd.h>
 #include <stdlib.h>
@@ -16,16 +39,18 @@
 #include <QModelIndex>
 #include <QScrollArea>
 #include <QMaemo5InformationBox>
+#include <QSettings>
 
 #include "mainwindow.h"
 #include "ui_mainwindow.h"
 #include "texteditautoresizer.h"
 #include "shareProgram/logindialog.h"
+#include "ConfigDialog.h"
 
 MainWindow::MainWindow(int argc,char *argv[],QWidget *parent)
     : QMainWindow(parent),
-      ui(new Ui::MainWindow),
-      m_fsIcon(FS_BUTTON_PATH)
+    ui(new Ui::MainWindow),
+    m_fsIcon(FS_BUTTON_PATH)
 {
     ui->setupUi(this);
     this->showFullScreen();
@@ -46,15 +71,14 @@ MainWindow::MainWindow(int argc,char *argv[],QWidget *parent)
 
     ui->tabWidget->removeTab(0);
     ui->actionFullScreen->setIcon(m_fsIcon);
-    ui->breakLineEdit->hide();
     /*!
      * Intialising debug toolbar
      */
     ui->mainToolBar->addAction(ui->actionFullScreen);
     ui->mainToolBar->addAction(ui->actionStopDebug);
     ui->mainToolBar->addAction(ui->actionContinue);
-    ui->mainToolBar->addAction(ui->actionBreakpoint);
     ui->mainToolBar->addAction(ui->actionWatchWindow);
+    ui->mainToolBar->addAction(ui->actionNext);
     ui->mainToolBar->addAction(ui->actionStep);
     ui->mainToolBar->addAction(ui->actionCompile);
     ui->mainToolBar->addAction(ui->actionRun);
@@ -101,11 +125,11 @@ MainWindow::MainWindow(int argc,char *argv[],QWidget *parent)
     connect(ui->actionCompile,SIGNAL(triggered()),this,SLOT(compile()));
     connect(ui->actionRun,SIGNAL(triggered()),this,SLOT(run()));
     connect(ui->actionStep,SIGNAL(triggered()),this,SLOT(debug()));
-    connect(ui->actionBreakpoint,SIGNAL(triggered()),this,SLOT(debug()));
+    connect(ui->actionNext,SIGNAL(triggered()),this,SLOT(debug()));
     connect(ui->actionContinue,SIGNAL(triggered()),this,SLOT(debug()));
     connect(ui->actionStopDebug,SIGNAL(triggered()),this,SLOT(debug()));
     connect(ui->actionAbout,SIGNAL(triggered()),this,SLOT(showAbout()));
-    connect(ui->breakLineEdit,SIGNAL(returnPressed()),this,SLOT(addBreakPoint()));
+    connect(ui->actionSettings,SIGNAL(triggered()),this,SLOT(showSettingsDialog()));
 
     connect(&gdbOutput,SIGNAL(updateLineNumber(QString)),this,SLOT(updateLineNumber(QString)));
     connect(&scanfThread,SIGNAL(stopDebug()),this,SLOT(stopDebug()));
@@ -137,14 +161,14 @@ MainWindow::MainWindow(int argc,char *argv[],QWidget *parent)
     init();
     fileNew();
 
-        if(argc>1) {
-            for(int i=1;i<argc;i++) {
-                open(QString(QDir::currentPath()+"/"+argv[i]));
-            }
-        }
-        else {
-            QDir::setCurrent("/home/user");
+    if(argc>1) {
+        for(int i=1;i<argc;i++) {
+            open(QFileInfo(QString(argv[i])).absoluteFilePath());
         }
+    }
+    else {
+        QDir::setCurrent("/home/user");
+    }
 }
 
 void MainWindow::init()
@@ -221,16 +245,16 @@ void MainWindow::rightButtonClicked(const QPoint &pos)
 void MainWindow::addWatchItem()
 {
     bool ok;
-         QString text = QInputDialog::getText(this, tr("Watch"),
-                                              tr("Variable Name:"), QLineEdit::Normal,
-                                              "", &ok);
-         if (ok && !text.isEmpty())
-         {
-            QString temp="disp "+ text+"\n";
-            ::write(flags.wr,temp.toAscii().data(),strlen(temp.toAscii().data()));
-            watchVariablesList<<text;
-            ui->watchWindow->addItem(text+" = ");
-         }
+    QString text = QInputDialog::getText(this, tr("Watch"),
+                                         tr("Variable Name:"), QLineEdit::Normal,
+                                         "", &ok);
+    if (ok && !text.isEmpty())
+    {
+        QString temp="disp "+ text+"\n";
+        ::write(flags.wr,temp.toAscii().data(),strlen(temp.toAscii().data()));
+        watchVariablesList<<text;
+        ui->watchWindow->addItem(text+" = ");
+    }
 }
 
 void MainWindow::deleteWatchItem()
@@ -428,6 +452,7 @@ bool MainWindow::maybeSave(int i)
         if(i==compiledTab || i==currentFileTab) {
             return false;
         }
+        return false;
     }
 
     if (!displayArea.at(i)->document()->isModified()) {
@@ -437,10 +462,10 @@ bool MainWindow::maybeSave(int i)
 
     QMessageBox::StandardButton ret;
     ret = QMessageBox::warning(this, tr("Application"),
-                                       "The document "+ fileName.at(i) +" has been modified.\n"
-                                       "Do you want to save your changes?",
-                                    QMessageBox::Save | QMessageBox::Discard
-                                    | QMessageBox::Cancel);
+                               "The document "+ QFileInfo(fileName.at(i)).fileName() +" has been modified.\n"
+                               "Do you want to save your changes?",
+                               QMessageBox::Save | QMessageBox::Discard
+                               | QMessageBox::Cancel);
 
     if (ret == QMessageBox::Save) {
         ui->tabWidget->setCurrentIndex(i);
@@ -451,63 +476,68 @@ bool MainWindow::maybeSave(int i)
 
     closeTab(i);
     return true;
- }
+}
 
-bool MainWindow::fileSave()
- {
+bool MainWindow::fileSave(bool diffFile)
+{
     if(ui->tabWidget->count()==0)
         return false;
 
     int currentTab=ui->tabWidget->currentIndex();
+    QString saveFilePath="";
 
-    if(displayArea.at(currentTab)->document()->isModified()) {
+    if(diffFile==true){
+        QString fn = QFileDialog::getSaveFileName(this, tr("Save as..."),
+                                                  QString(), "C File (*.c);;CPP files (*.cpp);;All Files (*)");
 
-        if (fileName.at(currentTab).compare("")==0)
-            return fileSaveAs();
-
-        QFile fileData(fileName.at(currentTab));
-        fileData.remove();
-
-        if (!fileData.open(QFile::WriteOnly | QFile::Truncate))
-           return false;
-
-        QTextStream out(&fileData);
-        out <<displayArea.at(currentTab)->toPlainText();
-        fileData.close();
-
-        displayArea.at(currentTab)->document()->setModified(false);
-        ui->tabWidget->setTabText(currentTab,ui->tabWidget->tabText(currentTab).mid(1));
-    }
-    return true;
- }
-
-bool MainWindow::fileSaveAs()
- {
-    if(ui->tabWidget->count()==0)
-        return false;
-
-    int currentTab=ui->tabWidget->currentIndex();
-     QString fn = QFileDialog::getSaveFileName(this, tr("Save as..."),
-                                               QString(), "C File (*.c);;CPP files (*.cpp);;All Files (*)");
-
-     if (fn.isEmpty())
-         return false;
-
-     if (! (fn.endsWith(".c", Qt::CaseInsensitive) || fn.endsWith(".cpp", Qt::CaseInsensitive) || fn.endsWith(".txt", Qt::CaseInsensitive)) )
-         fn += ".c";
-
-     QFile fileData(fn);
-     if (!fileData.open(QFile::WriteOnly | QFile::Truncate))
+        if (fn.isEmpty())
             return false;
+        if (! (fn.endsWith(".c", Qt::CaseInsensitive) || fn.endsWith(".cpp", Qt::CaseInsensitive) || fn.endsWith(".h", Qt::CaseInsensitive)) || fn.endsWith(".txt", Qt::CaseInsensitive))
+            fn += ".c";
+
+        saveFilePath=fn;
+    }
+    else{
+        if(displayArea.at(currentTab)->document()->isModified()) {
+            if (fileName.at(currentTab).compare("")==0){
+                QString fn = QFileDialog::getSaveFileName(this, tr("Save as..."),
+                                                          QString(), "C File (*.c);;CPP files (*.cpp);;All Files (*)");
+
+                if (fn.isEmpty())
+                    return false;
+
+                if (! (fn.endsWith(".c", Qt::CaseInsensitive) || fn.endsWith(".cpp", Qt::CaseInsensitive) || fn.endsWith(".h", Qt::CaseInsensitive)) || fn.endsWith(".txt", Qt::CaseInsensitive))
+                    fn += ".c";
+
+                saveFilePath=fn;
+                ui->tabWidget->setTabText(currentTab,QFileInfo(fn).fileName().mid(0,4));
+                fileName.replace(currentTab,fn);
+            }
+            else{
+                ui->tabWidget->setTabText(currentTab,ui->tabWidget->tabText(currentTab).mid(1));
+                saveFilePath=fileName.at(currentTab);
+            }
+            displayArea.at(currentTab)->document()->setModified(false);
+        }
+    }
+
+    QFile fileData(saveFilePath);
+    fileData.remove();
+
+    if (!fileData.open(QFile::WriteOnly | QFile::Truncate))
+        return false;
 
     QTextStream out(&fileData);
-    out <<displayArea[currentTab]->toPlainText();
+    out <<displayArea.at(currentTab)->toPlainText();
+    fileData.close();
 
-    fileName.removeAt(ui->tabWidget->currentIndex());
-    fileName.insert(ui->tabWidget->currentIndex(),fn);
-    ui->tabWidget->setTabText(currentTab,fn.mid(fn.lastIndexOf("/"),5));
-     return fileSave();
- }
+    return true;
+}
+
+bool MainWindow::fileSaveAs()
+{
+    fileSave(true);
+}
 
 void MainWindow::open(QString fname)
 {
@@ -518,43 +548,41 @@ void MainWindow::open(QString fname)
     QString selectedFileName;
 
     if(fname.isEmpty())
-        selectedFileName = QFileDialog::getOpenFileName(this,tr("Open File"), QDir::currentPath(),"*.c *.cpp *.cc *.txt");
+        selectedFileName = QFileDialog::getOpenFileName(this,tr("Open File"), QDir::currentPath(),"*.c *.cpp *.cc *.txt *.h");
     else
         selectedFileName=fname;
 
     QFile file(selectedFileName);
-     if (!file.open(QFile::ReadOnly | QFile::Text))
-         return;
+    if (!file.open(QFile::ReadOnly | QFile::Text))
+        return;
 
-     int tabCount=ui->tabWidget->count();
-
-     int currentTab=ui->tabWidget->currentIndex();
-     /*!
+    int currentTab=ui->tabWidget->currentIndex();
+    /*!
       * check whether the current document is modified or its already a new tab
       */
-     if(displayArea.at(currentTab)->document()->isModified() || fileName.at(currentTab).compare("")!=0)
-         fileNew();
+    if(displayArea.at(currentTab)->document()->isModified() || fileName.at(currentTab).compare("")!=0)
+        fileNew();
 
-     ui->tabWidget->setTabText(ui->tabWidget->currentIndex(),selectedFileName.mid(selectedFileName.lastIndexOf("/")+1,4));
+    ui->tabWidget->setTabText(ui->tabWidget->currentIndex(),QFileInfo(selectedFileName).fileName().mid(0,4));
 
-     currentTab=ui->tabWidget->currentIndex();
+    currentTab=ui->tabWidget->currentIndex();
 
-     /*!
+    /*!
       * set text of file, to the current tab
       */
-     fileName.removeAt(currentTab);
-     fileName.insert(currentTab,selectedFileName);
+    fileName.removeAt(currentTab);
+    fileName.insert(currentTab,selectedFileName);
 
-     QTextStream in(&file);
-     QApplication::setOverrideCursor(Qt::WaitCursor);
-     disconnect(displayArea.at(currentTab),SIGNAL(textChanged()),this,SLOT(documentModified()));
-     displayArea.at(ui->tabWidget->currentIndex())->setPlainText(in.readAll());
-     connect(displayArea.at(currentTab),SIGNAL(textChanged()),this,SLOT(documentModified()));
-     QApplication::restoreOverrideCursor();
- }
+    QTextStream in(&file);
+    QApplication::setOverrideCursor(Qt::WaitCursor);
+    disconnect(displayArea.at(currentTab),SIGNAL(textChanged()),this,SLOT(documentModified()));
+    displayArea.at(ui->tabWidget->currentIndex())->setPlainText(in.readAll());
+    connect(displayArea.at(currentTab),SIGNAL(textChanged()),this,SLOT(documentModified()));
+    QApplication::restoreOverrideCursor();
+}
 
 void MainWindow::fileNew()
- {
+{
     int tabCount=ui->tabWidget->count();
 
     QWidget *tempTab = new QWidget();
@@ -595,6 +623,7 @@ void MainWindow::fileNew()
     tempDisplayArea->clear();
     tempDisplayArea->setInputMethodHints(Qt::ImhNoAutoUppercase);
     connect(tempDisplayArea,SIGNAL(textChanged()),this,SLOT(documentModified()));
+    connect(tempDisplayArea,SIGNAL(setBreakPoint(int,bool)),this,SLOT(setBreakPoint(int,bool)));
 
     tab.append(tempTab);
     highlighter.append(tempHighlighter);
@@ -606,47 +635,48 @@ void MainWindow::fileNew()
     scrollAreaWidgetContents.append(tempScrollAreaWidgetContents);
 
     ui->tabWidget->setCurrentIndex(tabCount);
- }
+}
 
 void MainWindow::closeTab(int index)
 {
-        ui->tabWidget->removeTab(index);
-        /*!
+    disconnect(displayArea.at(index),SIGNAL(setBreakPoint(int,bool)),this,SLOT(setBreakPoint(int,bool)));
+    ui->tabWidget->removeTab(index);
+    /*!
          * as the display area document is passed in the constuctor deleting the parents lead to the delete of child
          * hence highlighter should not be deleted
          * the distructor of the highlighter is called by display area it self
          */
-        highlighter.removeAt(index);
+    highlighter.removeAt(index);
 
-        delete displayArea.at(index);
-        displayArea.removeAt(index);
+    delete displayArea.at(index);
+    displayArea.removeAt(index);
 
-        delete gridLayout2.at(index);
-        gridLayout2.removeAt(index);
+    delete gridLayout2.at(index);
+    gridLayout2.removeAt(index);
 
-        delete scrollAreaWidgetContents.at(index);
-        scrollAreaWidgetContents.removeAt(index);
+    delete scrollAreaWidgetContents.at(index);
+    scrollAreaWidgetContents.removeAt(index);
 
-        delete scrollArea.at(index);
-        scrollArea.removeAt(index);
+    delete scrollArea.at(index);
+    scrollArea.removeAt(index);
 
-        delete gridLayout.at(index);
-        gridLayout.removeAt(index);
+    delete gridLayout.at(index);
+    gridLayout.removeAt(index);
 
-        delete tab.at(index);
-        tab.removeAt(index);
+    delete tab.at(index);
+    tab.removeAt(index);
 
-        fileName.removeAt(index);
+    fileName.removeAt(index);
 
-        if(ui->tabWidget->count()==0)
-            this->close();
+    if(ui->tabWidget->count()==0)
+        this->close();
 
 }
 
 void MainWindow::closeEvent(QCloseEvent *event)
- {
+{
     if(flags.debugging==true) {
-        QMaemo5InformationBox::information(0, "Program is in Debuggin mode stop it before closeing",QMaemo5InformationBox::DefaultTimeout);
+        QMaemo5InformationBox::information(0, "Program is in Debuggin mode stop before closing",QMaemo5InformationBox::DefaultTimeout);
         event->ignore();
         return;
     }
@@ -654,6 +684,7 @@ void MainWindow::closeEvent(QCloseEvent *event)
     bool closeFlag=true;
 
     QFile fileData("/home/user/.edicom");
+    fileData.remove();
     fileData.open(QFile::WriteOnly | QFile::Truncate);
 
 
@@ -679,17 +710,17 @@ void MainWindow::closeEvent(QCloseEvent *event)
 
     }
 
-     if (closeFlag) {
+    if (closeFlag) {
         event->accept();
-     } else {
-         event->ignore();
-     }
+    } else {
+        event->ignore();
+    }
 
 
- }
+}
 
 void MainWindow::compile()
- {
+{
     if(ui->tabWidget->count()==0)
         return;
 
@@ -705,22 +736,33 @@ void MainWindow::compile()
 
     int currentTab=ui->tabWidget->currentIndex();
     QFile data(".temp.c");
-        if (data.open(QFile::WriteOnly | QFile::Truncate)) {
-            QTextStream out(&data);
-            QString str=displayArea.at(currentTab)->toPlainText();
-            if(str.contains("fcntl.h")==false)
-            {
-                disconnect(displayArea.at(currentTab),SIGNAL(textChanged()),this,SLOT(documentModified()));
-                displayArea.at(currentTab)->setPlainText("#include<fcntl.h>\n"+str);
-                connect(displayArea.at(currentTab),SIGNAL(textChanged()),this,SLOT(documentModified()));
-                str="#include<fcntl.h>\n"+str;
-            }
-            int ind=str.indexOf("main()\n{",0);
-            str.insert(ind+8,"setvbuf(stdout,NULL,_IONBF,0);int fdesc=open(\"/tmp/.PIN\",O_RDONLY);close(0);if(fcntl(fdesc,F_DUPFD,0)==-1){printf(\"counldn\");}fdesc=open(\"/tmp/.POUT\",O_WRONLY);close(1);if(fcntl(fdesc,F_DUPFD,1)==-1){printf(\"counldn\");}FILE *f=fopen(\"/tmp/pid.ec\",\"w\");fprintf(f,\"%d\",getpid());fclose(f);");
-            out <<str;
-         }
+    if (data.open(QFile::WriteOnly | QFile::Truncate)) {
+        QTextStream out(&data);
+        QString str=displayArea.at(currentTab)->toPlainText();
+        if(str.contains("fcntl.h")==false)
+        {
+            disconnect(displayArea.at(currentTab),SIGNAL(textChanged()),this,SLOT(documentModified()));
+            displayArea.at(currentTab)->setPlainText("#include<fcntl.h>\n"+str);
+            connect(displayArea.at(currentTab),SIGNAL(textChanged()),this,SLOT(documentModified()));
+            str="#include<fcntl.h>\n"+str;
+        }
 
-    if(system("gcc -g -o .temp .temp.c 2>.compiled.txt")==0)
+        int ind=str.indexOf("{",str.indexOf("main("));
+        str.insert(ind+1,"setvbuf(stdout,NULL,_IONBF,0);int fdesc=open(\"/tmp/.PIN\",O_RDONLY);close(0);if(fcntl(fdesc,F_DUPFD,0)==-1){printf(\"counldn\");}fdesc=open(\"/tmp/.POUT\",O_WRONLY);close(1);if(fcntl(fdesc,F_DUPFD,1)==-1){printf(\"counldn\");}FILE *f=fopen(\"/tmp/pid.ec\",\"w\");fprintf(f,\"%d\",getpid());fclose(f);");
+        out <<str;
+    }
+
+    QString compileString="gcc -g -o .temp .temp.c";
+    QSettings settings("SJCE", "edicom");
+    settings.beginGroup("BuildSettings");
+    if(settings.value("enable").toBool()){
+        compileString+=" "+settings.value("build").toString();
+    }
+    settings.endGroup();
+    compileString+=" 2>.compiled.txt";
+
+
+    if(system(compileString.toAscii().data())==0)
     {
         flags.setCompiledStatus(true);
         this->compiledTab=currentTab;
@@ -729,13 +771,13 @@ void MainWindow::compile()
     }
     else
     {
-         QFile file(".compiled.txt");
-         file.open(QFile::ReadOnly | QFile::Text);
-         QTextStream in(&file);
-         QApplication::setOverrideCursor(Qt::WaitCursor);
-         QMessageBox::warning(this,tr("Compilation"), in.readAll());
-         QApplication::restoreOverrideCursor();
-         flags.setCompiledStatus(false);
+        QFile file(".compiled.txt");
+        file.open(QFile::ReadOnly | QFile::Text);
+        QTextStream in(&file);
+        QApplication::setOverrideCursor(Qt::WaitCursor);
+        QMessageBox::warning(this,tr("Compilation"), in.readAll());
+        QApplication::restoreOverrideCursor();
+        flags.setCompiledStatus(false);
     }
 }
 
@@ -755,25 +797,25 @@ void MainWindow::run()
         return;
     }
 
-     if(flags.getCompiledStatus() && flags.debugging==false) {
+    if(flags.getCompiledStatus() && flags.debugging==false) {
         system("killall .temp 2> /tmp/.wst");
         system("killall gdb 2> /tmp/.wst");
         flags.running=true;
 
-         if(flags.firstRun==false)
-             delete t;
-         this->hide();
-         t=new Terminal();
-         t->show();
-         connect(t,SIGNAL(terminalExiting()),this,SLOT(terminalExiting()));
-         flags.firstRun=false;
-     }
-     else
-         QMaemo5InformationBox::information(0, "Program has few errors",QMaemo5InformationBox::DefaultTimeout);
+        if(flags.firstRun==false)
+            delete t;
+        this->hide();
+        t=new Terminal();
+        t->show();
+        connect(t,SIGNAL(terminalExiting()),this,SLOT(terminalExiting()));
+        flags.firstRun=false;
+    }
+    else
+        QMaemo5InformationBox::information(0, "Program has few errors",QMaemo5InformationBox::DefaultTimeout);
 }
 
 void MainWindow::highlightDebuggingLine(int linenum,int color)
- {
+{
     ui->tabWidget->setCurrentIndex(this->currentFileTab);
 
     QTextBlock block=displayArea.at(currentFileTab)->firstVisibleBlk();
@@ -794,34 +836,22 @@ void MainWindow::highlightDebuggingLine(int linenum,int color)
         for(int i=0;i<linenum-first_vis_line;i++)
             block=block.next();
     }
-         QTextCursor cur(block);
-         QList<QTextEdit::ExtraSelection> extraSelections;
-         QTextEdit::ExtraSelection selection;
+    QTextCursor cur(block);
+    QList<QTextEdit::ExtraSelection> extraSelections;
+    QTextEdit::ExtraSelection selection;
 
-         QColor lineColor;
-         if(color==0)
-            lineColor = QColor(Qt::lightGray);
-         else
-            lineColor = QColor(Qt::white);
+    QColor lineColor;
+    if(color==0)
+        lineColor = QColor(Qt::lightGray);
+    else
+        lineColor = QColor(Qt::white);
 
-         selection.format.setBackground(lineColor);
-         selection.format.setProperty(QTextFormat::FullWidthSelection, true);
-         selection.cursor = cur;
-         selection.cursor.clearSelection();
-         extraSelections.append(selection);
-         displayArea.at(currentFileTab)->setExtraSelections(extraSelections);
- }
-
-void MainWindow::addBreakPoint()
-{
-    if(ui->tabWidget->count()==0)
-        return;
-
-    QString temp="b "+ui->breakLineEdit->text()+"\n";
-    if(ui->breakLineEdit->text()!="\n")
-        ::write(flags.wr,temp.toAscii().data(),temp.length());
-
-    ui->breakLineEdit->hide();
+    selection.format.setBackground(lineColor);
+    selection.format.setProperty(QTextFormat::FullWidthSelection, true);
+    selection.cursor = cur;
+    selection.cursor.clearSelection();
+    extraSelections.append(selection);
+    displayArea.at(currentFileTab)->setExtraSelections(extraSelections);
 }
 
 void MainWindow::debug()
@@ -864,20 +894,31 @@ void MainWindow::debug()
             while((line=dat.readLine(100))!="") {
                 QString temp=QString(line);
                 lineNum++;
-                if(temp.contains("main()"))
+                if(temp.contains("main(")){
+                    if(!temp.contains("{"))
+                        lineNum++;
                     break;
+                }
             }
             dat.close();
 
+            QString argvString="";
+            QSettings settings("SJCE", "edicom");
+            settings.beginGroup("BuildSettings");
+            if(settings.value("enable").toBool()){
+                argvString=settings.value("arguments").toString();
+            }
+            settings.endGroup();
+
             QFile fileData("/tmp/.a");
-            QString s="set logging file .gdb.txt\nset logging on\nstart\nb "+QString::number(lineNum+2)+"\nc\nbt\n";
+            QString s="set logging file .gdb.txt\nset logging on\nb "+QString::number(lineNum+1)+"\nset args "+argvString+"\nstart\nc\nbt\n";
             if (fileData.open(QFile::WriteOnly | QFile::Truncate)) {
                 QTextStream out(&fileData);
                 out <<s;
-             }
+            }
             fileData.close();
 
-//--------------------------------------------------------------------------------------------------------------------
+            //--------------------------------------------------------------------------------------------------------------------
 
             system("killall .temp 2> /tmp/.wst");
             system("killall gdb 2> /tmp/.wst");
@@ -904,6 +945,13 @@ void MainWindow::debug()
             }
             ::write(flags.wr,temporary.toAscii().data(),strlen(temporary.toAscii().data()));
 
+            for(int i=0;i<displayArea.length();i++){
+                QList<int> breakPoints=displayArea.at(i)->getBreakPoints();
+                for(int j=0;j<breakPoints.count();j++)
+                {
+                    setBreakPoint(breakPoints.at(j),true,fileName.at(i));
+                }
+            }
             return;
         }
         QAction *c=qobject_cast<QAction *>(sender());
@@ -918,10 +966,10 @@ void MainWindow::debug()
             strcpy(str,"c\nbt\n");
             ::write(flags.wr,str,strlen(str));
         }
-        if(menuClicked=="breakpoint")
+        if(menuClicked=="next")
         {
-            ui->breakLineEdit->show();
-            ui->breakLineEdit->focusWidget();
+            strcpy(str,"n\nbt\n");
+            ::write(flags.wr,str,strlen(str));
         }
         if(menuClicked=="stopdebug" && flags.debugging==true)
         {
@@ -946,24 +994,20 @@ void MainWindow::updateLineNumber(QString str)
 
             QStringList lineContent= line[i].split(" ");
             QStringList lineNum=lineContent[lineContent.length()-1].split(":");
-            QString file=lineNum[0];
+            QString file=QFileInfo(lineNum[0]).absoluteFilePath();
 
             /*!
              * check whether the extra selection moved to other file
              */
-            if(file.compare(".temp.c")!=0 || currentFileTab!=compiledTab) {
+            if(file.compare(QFileInfo(".temp.c").absoluteFilePath())!=0 || currentFileTab!=compiledTab) {
                 int k;
-                if(!file.startsWith("/") && file.compare(".temp.c")!=0) {
-                    file=QDir::currentPath()+"/"+file;
-                }
 
                 for(k=0;k<ui->tabWidget->count();k++) {
-                    qDebug()<<"not same file hiya "<<fileName.at(k)<<" : "<<QDir::currentPath()+"/"+file;
                     if(file.compare(fileName.at(k))==0)
                         break;
                 }
 
-                if(file.compare(".temp.c")==0){
+                if(file.compare(QFileInfo(".temp.c").absoluteFilePath())==0){
                     highlightDebuggingLine(0,1);
                     currentFileTab=compiledTab;
                     ui->tabWidget->setCurrentIndex(currentFileTab);
@@ -985,7 +1029,7 @@ void MainWindow::updateLineNumber(QString str)
                     highlightDebuggingLine(0,1);
                 }
             }
-//////////////////////////////////////////////////////////////
+            //////////////////////////////////////////////////////////////
             bool bo;
             int num=lineNum[1].toInt(&bo,10);
 
@@ -994,13 +1038,15 @@ void MainWindow::updateLineNumber(QString str)
                 stopDebug();
                 return;
             }
-                flags.debugging_line=num-1;
-                highlightDebuggingLine(flags.debugging_line);
+            flags.debugging_line=num-1;
+            highlightDebuggingLine(flags.debugging_line);
         }
         for(int j=0;j<watchVariablesList.length();j++)
         {
-            if(line[i].contains(QRegExp("[0-9]+[:][ ]"+watchVariablesList[j]))) {
-                ui->watchWindow->item(j)->setText(( line[i].mid(line[i].indexOf(watchVariablesList[j]+" = ")) ));
+            if(line[i].contains(QRegExp("^\\d+: "))) {
+                if(line[i].contains(watchVariablesList[j])) {
+                    ui->watchWindow->item(j)->setText(( line[i].mid(line[i].indexOf(watchVariablesList[j]+" = ")) ));
+                }
             }
         }
     }
@@ -1064,3 +1110,39 @@ void MainWindow::terminalExiting()
     this->show();
     flags.running=false;
 }
+
+void MainWindow::setBreakPoint(int lineNumber, bool setFlag, QString toFile)
+{
+    if(flags.debugging==false) {
+        return;
+    }
+    if(ui->tabWidget->count()==0)
+        return;
+
+    if(setFlag==true){
+        QString breakPointInFile;
+        if(toFile.isEmpty()){
+            breakPointInFile=QFileInfo(fileName.at(ui->tabWidget->currentIndex())).fileName()+":";
+        }
+        else
+            breakPointInFile=QFileInfo(toFile).fileName()+":";
+
+        if((toFile.isEmpty() && ui->tabWidget->currentIndex()==compiledTab) || toFile.compare(fileName.at(compiledTab))==0)
+            breakPointInFile=".temp.c:";
+
+        qDebug()<<"b "+breakPointInFile+QString::number(lineNumber+1)+"\n"<<" toFile:"+toFile;
+        QString temp="b "+breakPointInFile+QString::number(lineNumber+1)+"\n";
+        ::write(flags.wr,temp.toAscii().data(),temp.length());
+    }
+    else{
+        QString temp="clear "+QString::number(lineNumber+1)+"\n";
+        ::write(flags.wr,temp.toAscii().data(),temp.length());
+    }
+}
+
+void MainWindow::showSettingsDialog()
+{
+    ConfigDialog configDialog;
+    configDialog.exec();
+}
+
